MODULE Blinkers;
(* =========================================================================  
   Example ARM Cortex-M3 Oberon Program  
 
   Description:
     Led connected to P1.18 blinking approx. once per second.
     Uses two independent timers to illustrate four different ways to 
     implement a half-second delay.
     
   Target:
     NXP LPC1768
     
   Tested on: 
     ARM mbed
   
   References: 
     NXP UM10360 LPC17xx User manual
     Oberon for Cortex-M3 Microcontrollers
   
   (c) 2012-2017 CFB Software   
   http://www.astrobe.com  
   
   ========================================================================= *)

IMPORT Main, MCU, SYSTEM, Timers;

PROCEDURE Run();
CONST
  (* led connected to pin P1.18 *)
  ledBit = {18};
VAR
  direction: SET;
  msecTimer: Timers.Timer;
  usecTimer: Timers.Timer;
BEGIN
  Timers.Init(msecTimer, Timers.T0, Timers.MSecs);
  Timers.Init(usecTimer, Timers.T1, Timers.uSecs);
  (* Set led pin as output by setting the direction bit *)
  SYSTEM.GET(MCU.FIO1DIR, direction);
  SYSTEM.PUT(MCU.FIO1DIR, direction + ledBit);
  
  WHILE TRUE DO
    SYSTEM.PUT(MCU.FIO1SET, ledBit);
    Timers.Delay(msecTimer, 500);
    SYSTEM.PUT(MCU.FIO1CLR, ledBit);
    Timers.Delay(usecTimer, 500000);
    SYSTEM.PUT(MCU.FIO1SET, ledBit);
    Timers.Start(msecTimer);
    WHILE Timers.Elapsed(msecTimer) < 500 DO END;
    SYSTEM.PUT(MCU.FIO1CLR, ledBit);
    Timers.Start(usecTimer);
    WHILE Timers.Elapsed(usecTimer) < 500000 DO END;
  END
END Run;

BEGIN
  Run()
END Blinkers.
