MODULE Timers;
(* =========================================================================  
   Astrobe Library Functions for LPC175x/6x/8x multiple 32-bit Timers 0/1/2/3  
   
   Target: LPC175x/6x/8x family microcontrollers
   
   Refs: 
     NXP LPC176x/5x User Manual UM10360
     NXP LPC178x/7x User Manual UM10470

   (c) 2017 CFB Software   
   http://www.astrobe.com  
   
   ========================================================================= *)

IMPORT MCU, SYSTEM;

CONST
  disable = {};
  enable = {0};
  reset = {1};
  (* Possible values for units *)
  uSecs* = 1000000;
  MSecs* = 1000;
  (* Timers *)
  T0* = 0;
  T1* = 1;
  T2* = 2;
  T3* = 3;
  (* Timer registers *)
  TCR = 04H;
  TC  = 08H;
  PR  = 0CH;
 
TYPE
  Timer* = RECORD
    base, prescale, units: INTEGER
  END;

PROCEDURE* Prescale(divisor: INTEGER): INTEGER;
  (* Round up instead of truncating *)
  RETURN (MCU.PCLK + (divisor DIV 2)) DIV divisor
END Prescale;


PROCEDURE* Delay*(timer: Timer; delay: INTEGER);
VAR
  count, tcr, tc: INTEGER;
BEGIN
  IF delay > 0 THEN 
    tcr := timer.base + TCR;
    tc := timer.base + TC;
    SYSTEM.PUT(tcr, reset);
    SYSTEM.PUT(timer.base + PR, (timer.prescale * delay) - 1);
    SYSTEM.PUT(tcr, enable);
    REPEAT SYSTEM.GET(tc, count) UNTIL count # 0;
    SYSTEM.PUT(tcr, disable)
  ELSE
    ASSERT(delay = 0, 20)
  END
END Delay;


PROCEDURE Init*(VAR timer: Timer; timerNo, units: INTEGER);
BEGIN
  CASE timerNo OF 
    T0: timer.base := MCU.T0Base |
    T1: timer.base := MCU.T1Base |
    T2: timer.base := MCU.T2Base |
    T3: timer.base := MCU.T3Base 
  END;
  SYSTEM.PUT(timer.base + TCR, disable + reset);
  timer.prescale := Prescale(units);
  timer.units := units
END Init;


PROCEDURE* Start*(timer: Timer);
VAR
  tcr: INTEGER;
BEGIN
  tcr := timer.base + TCR;
  SYSTEM.PUT(tcr, reset);
  SYSTEM.PUT(timer.base + PR, timer.prescale - 1);
  SYSTEM.PUT(tcr, enable);
END Start;


PROCEDURE* Stop*(timer: Timer);
BEGIN
  SYSTEM.PUT(timer.base + TCR, disable);
END Stop;


PROCEDURE* Elapsed*(timer: Timer): INTEGER;
VAR
  count: INTEGER;
BEGIN
  SYSTEM.GET(timer.base + TC, count);
  RETURN(count)
END Elapsed;

END Timers.
  
  
